clear
set more off
set mem 2800m

adopath ++ "/public/programs/stata"

**** Due to memory constraints, I split this job in half.  Even and odd lbdnums are
**** run separately.  Be sure to assign the type variable to either "odd" or "even"
**** before running the program.

*local type = "01"
foreach type in "01" "23" "456" "789" {

local indata = "/rdcprojects/mc1/mc00535/data/geocoding/Address Data/02-LongitudinalImp/Long_Imp_Data_`type'.dta"
local outdata = "/rdcprojects/mc1/mc00535/data/geocoding/Address Data/02-LongitudinalImp/blocks_lbdnums_`type'"
local admindata = "/rdcprojects/mc1/mc00535/data/ezdata/blocklevel_admdata_sort00.dta"

clear
use "`indata'"

sort lbdnum yr

gen blockn = real(block)
drop block
ren blockn block

**** Must be double precision of last few digits become corrupted
gen double blockid=tract*1000000+block

**** Initialize a flag identifying any imputation done to the blockid variable
gen blockid_imp_f = 0

****
**** 1a) Impose that all instances of the same address string within establishment
****     have the same blockid assigned.  The chosen blockid will be the blockid with the
****     highest match _score_, with ties broken arbitrarily with "rank, unique"
****

**** Define id group, for example (for some test
**** runs it;s helpful to use "lbdnum beststreet")
egen idlocation = group(lbdnum beststreet)
drop beststreet

****
**** 1b) Impose that within an establishment, all observations with the same street
****     address string are coded to the blockid matched to that address with the 
****     highest "_score_".
****

******************************
**** Define the tempvars *****
******************************
tempvar coded numcoded evercoded addr_sc_rank addr_best_blockid

**** identify if the current year;s address was geocoded
gen `coded' = blockid!=.

**** count number of years in which a blockid variable was assigned within establishment
egen `numcoded' = sum(`coded'), by(idlocation)

**** create a flag indicating if the establishment was ever coded
gen `evercoded' = `numcoded'>0

****Populate the _score_ variable for non-coded addresses with -9
replace _score_ = -9 if _score_ == .

**** Create a variable ranking the _score_ variable within unique establishment address
egen `addr_sc_rank' = rank(-_score_), unique by(idlocation)
egen double `addr_best_blockid' = sum( (`addr_sc_rank' == 1) * blockid ) if `evercoded' == 1, by(idlocation)


**** - Imputation Step - force all instances of the same street address
****                     within and establishment to have the best blockid
****                     assigned to that blockid
replace blockid_imp_f = 1 if blockid != `addr_best_blockid'
replace blockid = `addr_best_blockid' if blockid != `addr_best_blockid'

drop `coded' `numcoded' `evercoded' `addr_best_blockid' `addr_best_blockid'

****
**** 2) Impose that if a blockid is ever assigned, all leading missing blockids are set
****    to the value of the first non-missing blockid.
****

******************************
**** Define the tempvars *****
******************************
tempvar est_obsnum first_matchobs first_blockid

bysort lbdnum: gen `est_obsnum' = _n
egen `first_matchobs' = min( (blockid!=.) * `est_obsnum' + (blockid==.) * 99), by(lbdnum)
replace `first_matchobs' = -99 if `first_matchobs' == 99
egen `first_blockid' = sum( blockid * (`est_obsnum' == `first_matchobs') ) if `first_matchobs' > 0, by(lbdnum)

**** - Imputation Step - If the first nonmissing blockid occurs in the t_th
****                     observation for an establishment, set the first
****                     t-1 blockid variables to the value of the t_th
replace blockid_imp_f = 2 if `est_obsnum' < `first_matchobs'
replace blockid = `first_blockid' if `est_obsnum' < `first_matchobs'
drop `est_obsnum' `first_matchobs' `first_blockid'


****
**** 3) Impose that if a blockid is ever assigned, all trailing missing blockids are set
****    to the value of the last non-missing blockid.
****

******************************
**** Define the tempvars *****
******************************
tempvar est_obsnum last_matchobs last_blockid

bysort lbdnum: gen `est_obsnum' = _n
egen `last_matchobs' = max( (blockid!=.) * `est_obsnum' + (blockid==.) * -99), by(lbdnum)
replace `last_matchobs' = 99 if `last_matchobs' == -99
egen `last_blockid' = sum( blockid * (`est_obsnum' == `last_matchobs') ) if `last_matchobs' < 99, by(lbdnum)

**** - Imputation Step - If the first nonmissing blockid occurs in the t_th
****                     observation for an establishment, set the first
****                     t-1 blockid variables to the value of the t_th
replace blockid_imp_f = 3 if `est_obsnum' > `last_matchobs'
replace blockid = `last_blockid' if `est_obsnum' > `last_matchobs'

drop `est_obsnum' `last_matchobs' `last_blockid'

egen id = group(lbdnum)
global browsevars = "id yr idlocation bestzip5 _score_ blockid matchtypenum est_obsnum first_matchobs first_blockid"


****
**** 4) When a gap appears in the sequence of assigned blockids, fill in the gap by
****    by assigning the first half of the gap to the last non-missing blockid before
****    the gap and assigning the second half of the gap to the first non-missing
****    blockid after the gap.  If there are an odd number of periods in the
****    gap, group the middle period with the first half with probability .5 and
****    with the second half with proability .5
****

**** - Imputation Step - 
forval ii = 1/10 {
by lbdnum: replace blockid_imp_f = 4     if blockid == . & blockid[_n+1] != . & (`ii'>1 | uniform()<.5)
by lbdnum: replace blockid = blockid[_n+1] if blockid == . & blockid_imp_f == 4

by lbdnum: replace blockid_imp_f = 4     if blockid == . & blockid[_n-1] != .
by lbdnum: replace blockid = blockid[_n-1] if blockid == . & blockid_imp_f == 4
}

**** Convert fips, blockid, and block to string variables
format blockid %10.0f
gen blockid_char = string(blockid,"%10.0f")
replace blockid_char = "" if blockid_char == "."

replace newfips = "12086" if newfips == "12025"

forval i = 1/6 {
replace blockid_char = "0" + blockid_char if length(blockid_char)<10 & blockid_char != ""
}

**** Create a character string containing the concatination of fips and blockid
drop blockid
gen blockid = newfips+blockid_char if blockid_char != ""
gen tractid = substr(blockid,1,11) if blockid_char != ""

keep lbdnum yr tractid blockid matchtypenum blockid_imp_f

savasas using "`outdata'.sas7bdat", replace

}

